@@ -30,10 +30,12 @@ from zope.container.contained import NameChooser
 from zope.container.interfaces import INameChooser
 from zope.event import notify
 from zope.i18n import translate
+from zope.i18n.interfaces.locales import ICollator
 from zope.interface import implements, Invalid, directlyProvides
 from zope.intid.interfaces import IIntIds
 from zope.publisher.browser import BrowserView
 from zope.publisher.interfaces.browser import IBrowserRequest
+from zope.cachedescriptors.property import Lazy
 from zope.schema import Choice
 from zope.schema import ValidationError
 from zope.security.checker import canAccess
@@ -1086,13 +1088,15 @@ class FlourishSectionView(DisplayForm):
             if not widget.value:
                 widget.mode = HIDDEN_MODE
 
-    @property
+    @Lazy
     def learners_table(self):
-        return self.getTable(list(self.context.members), 'students')
+        return self.getTable(list(self.context.members), 'students',
+                             batch_size=0)
 
-    @property
+    @Lazy
     def instructors_table(self):
-        return self.getTable(list(self.context.instructors), 'instructors')
+        return self.getTable(list(self.context.instructors), 'instructors',
+                             batch_size=0)
 
     def has_instructors(self):
         return bool(list(self.context.instructors))
@@ -1100,11 +1104,11 @@ class FlourishSectionView(DisplayForm):
     def has_learners(self):
         return bool(list(self.context.members))
 
-    def getTable(self, items, prefix):
+    def getTable(self, items, prefix, **kw):
         persons = ISchoolToolApplication(None)['persons']
         result = getMultiAdapter((persons, self.request), ITableFormatter)
         result.setUp(table_formatter=table.StandaloneFullFormatter, items=items,
-                     prefix=prefix)
+                     prefix=prefix, **kw)
         return result
 
 
@@ -1171,7 +1175,7 @@ class FlourishSectionAddView(Form, SectionAddView):
     def nextURL(self):
         return absoluteURL(self._section, self.request)
 
-    @button.buttonAndHandler(_('Add'), name='add')
+    @button.buttonAndHandler(_('Submit'), name='add')
     def handleAdd(self, action):
         data, errors = self.extractData()
         if errors:
@@ -1229,6 +1233,23 @@ WidgetValidatorDiscriminators(SectionTermsValidator,
 
 class FlourishNewSectionCoursesSubform(NewSectionCoursesSubform):
 
+    def __init__(self, *args, **kw):
+        default_course = kw.pop('default_course', None)
+        subform.EditSubForm.__init__(self, *args, **kw)
+        courses = ICourseContainer(self.context)
+        collator = ICollator(self.request.locale)
+        items = sorted(courses.values(),
+                       cmp=collator.cmp,
+                       key=lambda course:course.title)
+        self.vocabulary=vocabulary_titled(items)
+        self.values = {'course': default_course}
+        schema_field = Choice(
+            __name__='course', title=_('Course'),
+            required=True, vocabulary=self.vocabulary)
+        self.fields += field.Fields(schema_field)
+        datamanager.DictionaryField(self.values, schema_field)
+
+
     def updateWidgets(self):
         super(FlourishNewSectionCoursesSubform, self).updateWidgets()
         self.widgets['course'].prompt = True
@@ -1332,6 +1353,16 @@ class FlourishSectionLearnerView(FlourishRelationshipViewBase):
     current_title = _("Current students")
     available_title = _("Add students")
 
+    def setUpTables(self):
+        self.available_table = self.createTableFormatter(
+            ommit=self.getOmmitedItems(),
+            prefix="add_item")
+
+        self.selected_table = self.createTableFormatter(
+            filter=lambda l: l,
+            items=self.getSelectedItems(),
+            prefix="remove_item")
+
     def getSelectedItems(self):
         return filter(IPerson.providedBy, self.context.members)
 
