@@ -23,44 +23,53 @@ import xlrd
 import transaction
 import datetime
 
-from zope.i18n import translate
+from zope.container.contained import containedEvent
 from zope.container.interfaces import INameChooser
+from zope.event import notify
 from zope.component import queryUtility
 from zope.security.proxy import removeSecurityProxy
 from zope.publisher.browser import BrowserView
+from zope.traversing.browser.absoluteurl import absoluteURL
 
+import schooltool.skin.flourish.page
 from schooltool.basicperson.interfaces import IDemographicsFields
 from schooltool.basicperson.interfaces import IDemographics
 from schooltool.basicperson.demographics import DateFieldDescription
+from schooltool.basicperson.demographics import BoolFieldDescription
 from schooltool.basicperson.person import BasicPerson
+from schooltool.contact.contact import Contact, ContactPersonInfo
+from schooltool.contact.interfaces import IContact, IContactContainer
+from schooltool.contact.interfaces import IContactPersonInfo, IContactable
 from schooltool.resource.resource import Resource
 from schooltool.resource.resource import Location
+from schooltool.resource.resource import Equipment
 from schooltool.group.group import Group
 from schooltool.group.interfaces import IGroupContainer
-from schooltool.timetable import TimetableActivity
-from schooltool.timetable import SchooldaySlot
-from schooltool.timetable import SchooldayTemplate
-from schooltool.timetable.browser import parse_time_range
-from schooltool.timetable.interfaces import ITimetableCalendarEvent
-from schooltool.timetable.interfaces import ITimetables
-from schooltool.timetable.interfaces import ITimetableSchemaContainer
-from schooltool.timetable.interfaces import ITimetableModelFactory
-from schooltool.timetable.schema import TimetableSchema
-from schooltool.timetable.schema import TimetableSchemaDay
 from schooltool.term.interfaces import ITerm
 from schooltool.term.term import Term, getNextTerm, getPreviousTerm
 from schooltool.app.interfaces import ISchoolToolApplication
-from schooltool.app.interfaces import ISchoolToolCalendar
+from schooltool.app.interfaces import IApplicationPreferences
 from schooltool.app.app import SimpleNameChooser
 from schooltool.schoolyear.schoolyear import SchoolYear
 from schooltool.schoolyear.interfaces import ISchoolYear
 from schooltool.schoolyear.interfaces import ISchoolYearContainer
-
+from schooltool.skin import flourish
 from schooltool.course.section import Section
 from schooltool.course.interfaces import ISectionContainer
 from schooltool.course.interfaces import ICourseContainer
 from schooltool.course.course import Course
 from schooltool.common import DateRange
+from schooltool.common import parse_time_range
+from schooltool.timetable.daytemplates import CalendarDayTemplates
+from schooltool.timetable.daytemplates import WeekDayTemplates
+from schooltool.timetable.daytemplates import SchoolDayTemplates
+from schooltool.timetable.daytemplates import DayTemplate
+from schooltool.timetable.daytemplates import TimeSlot
+from schooltool.timetable.interfaces import ITimetableContainer
+from schooltool.timetable.interfaces import IScheduleContainer
+from schooltool.timetable.schedule import Period
+from schooltool.timetable.timetable import Timetable
+from schooltool.timetable.timetable import SelectedPeriodsSchedule
 
 from schooltool.common import format_message
 from schooltool.common import SchoolToolMessage as _
@@ -87,18 +96,22 @@ ERROR_DUPLICATE_DAY_ID = _("is the same day id as another in this timetable")
 ERROR_UNKNOWN_DAY_ID = _("is not defined in the 'Day Templates' section")
 ERROR_DUPLICATE_PERIOD = _("is the same period id as another in this day")
 ERROR_DUPLICATE_HOMEROOM_PERIOD = _("is the same homeroom period id as another in this day")
-ERROR_RESOURCE_TYPE = _("must be either 'Location' or 'Resource'")
+ERROR_RESOURCE_TYPE = _("must be either 'Location', 'Equipment' or 'Resource'")
 ERROR_INVALID_TERM_ID = _('is not a valid term in the given school year')
 ERROR_INVALID_COURSE_ID = _('is not a valid course in the given school year')
 ERROR_HAS_NO_COURSES = _('${title} has no courses in A${row}')
 ERROR_INVALID_PERSON_ID = _('is not a valid username')
-ERROR_INVALID_SCHEMA_ID = _('is not a valid schema in the given school year')
-ERROR_INVALID_DAY_ID = _('is not a valid day id for the given schema')
+ERROR_INVALID_SCHEMA_ID = _('is not a valid timetable in the given school year')
+ERROR_INVALID_DAY_ID = _('is not a valid day id for the given timetable')
 ERROR_INVALID_PERIOD_ID = _('is not a valid period id for the given day')
+ERROR_INVALID_CONTACT_ID = _('is not a valid username or contact id')
+ERROR_UNWANTED_CONTACT_DATA = _('must be empty when ID is a user id')
 ERROR_INVALID_RESOURCE_ID = _('is not a valid resource id')
 ERROR_UNICODE_CONVERSION = _(
     "Username cannot contain non-ascii characters: ${string}")
 ERROR_WEEKLY_DAY_ID = _('is not a valid weekday number (0-6)')
+ERROR_CONTACT_RELATIONSHIP = _("is not a valid contact relationship")
+ERROR_NOT_BOOLEAN = _("must be either True or False")
 
 
 no_date = object()
@@ -164,13 +177,12 @@ class ImporterBase(object):
         value, found = self.getCellAndFound(sheet, row, col, default)
         valid = True
         if found:
-            if isinstance(value, str):
-                try:
-                    value = unicode(value)
-                except UnicodeError:
-                    self.error(row, col, ERROR_NOT_UNICODE_OR_ASCII)
-                    valid = False
-            elif not isinstance(value, unicode):
+            if isinstance(value, float):
+                if int(value) == value:
+                    value = int(value)
+            try:
+                value = unicode(value)
+            except UnicodeError:
                 self.error(row, col, ERROR_NOT_UNICODE_OR_ASCII)
                 valid = False
         return value, found, valid
@@ -186,16 +198,59 @@ class ImporterBase(object):
         return value
 
     def getDateFromCell(self, sheet, row, col, default=no_date):
-        try:
-            dt = xlrd.xldate_as_tuple(sheet.cell_value(rowx=row, colx=col), self.wb.datemode)
-        except:
-            if default is not no_date:
-                return default
-            else:
+        value, found = self.getCellAndFound(sheet, row, col)
+        if not found or value == '':
+            if default is no_date:
                 self.error(row, col, ERROR_NO_DATE)
                 return None
+            else:
+                return default
+        try:
+            dt = xlrd.xldate_as_tuple(value, self.wb.datemode)
+        except:
+            self.error(row, col, ERROR_NO_DATE)
+            return None
         return datetime.datetime(*dt).date()
 
+    def getBoolFromCell(self, sheet, row, col):
+        value, found = self.getCellAndFound(sheet, row, col)
+        if not found:
+            return None
+        if type(value) == type(True):
+            return value
+        if type(value) == type(0):
+            return bool(value)
+        value, found, valid = self.getTextFoundValid(sheet, row, col)
+        if not valid or not value:
+            return None
+        if value.upper() == 'TRUE':
+            return True
+        elif value.upper() == 'FALSE':
+            return False
+        else:
+            self.error(row, col, ERROR_NOT_BOOLEAN)
+            return None
+
+    def getRequiredBoolFromCell(self, sheet, row, col):
+        value, found = self.getCellAndFound(sheet, row, col)
+        if not found or value == '':
+            self.error(row, col, ERROR_MISSING_REQUIRED_TEXT)
+            return None
+        return self.getBoolFromCell(sheet, row, col)
+
+    def validateUnicode(self, value, row, col):
+        # XXX: this has to be fixed
+        # XXX: SchoolTool should handle UTF-8
+        try:
+            value.encode('ascii')
+        except UnicodeEncodeError:
+            self.error(
+                row, col,
+                format_message(
+                    ERROR_UNICODE_CONVERSION,
+                    mapping={'string': value})
+                )
+
     @property
     def sheet(self):
         if self.sheet_name not in self.wb.sheet_names():
@@ -376,65 +431,101 @@ class SchoolTimetableImporter(ImporterBase):
     dows = ['Monday', 'Tuesday', 'Wednesday', 'Thursday',
             'Friday', 'Saturday', 'Sunday']
 
-
-    def setUpSchemaDays(self, timetable, days):
-        for day in days:
-            day_id = day['id']
-            timetable[day_id] = TimetableSchemaDay(day['periods'], 
-                                                   day['homeroom_periods'])
+    day_templates = (
+        ('calendar_days', CalendarDayTemplates),
+        ('week_days', WeekDayTemplates),
+        ('school_days', SchoolDayTemplates),
+        )
 
     def createSchoolTimetable(self, data):
-        title = data['title']
-        factory = queryUtility(ITimetableModelFactory, data['model'])
-        day_ids = [day['id'] for day in data['days']]
-
-        template_dict = {}
-
-        for template in data['templates']:
-            used = template['id']
-
-            # parse SchoolDayPeriods
-            day = SchooldayTemplate()
-            for tstart, duration in template['periods']:
-                slot = SchooldaySlot(tstart, duration)
-                day.add(slot)
-
-            if used == 'default':
-                template_dict[None] = day
-            elif used in day_ids:
-                template_dict[used] = day
-            else:
-                try:
-                    template_dict[self.dows.index(used)] = day
-                except ValueError:
-                    self.errors.append("Unrecognised day id %s" % used)
-
-        model = factory(day_ids, template_dict)
-
-        # create and set up the timetable
-        timetable = TimetableSchema(day_ids, title=title, model=model)
-        self.setUpSchemaDays(timetable, data['days'])
-        timetable.__name__ = data['__name__']
-        return timetable
-
-    def addSchoolTimetable(self, school_timetable, data):
         syc = ISchoolYearContainer(self.context)
-        sy = syc[data['school_year']]
-        sc = ITimetableSchemaContainer(sy)
-        if school_timetable.__name__ is None:
-            school_timetable.__name__ = SimpleNameChooser(sc).chooseName('', school_timetable)
+        schoolyear = syc[data['school_year']]
 
-        if school_timetable.__name__ not in sc:
-            sc[school_timetable.__name__] = school_timetable
+        app = ISchoolToolApplication(None)
+        tzname = IApplicationPreferences(app).timezone
+        timetable = Timetable(schoolyear.first, schoolyear.last,
+                              title=data['title'], timezone=tzname)
+
+        factories = dict(self.day_templates)
+
+        container = ITimetableContainer(schoolyear)
+
+        if data['__name__'] in container:
+            del container[data['__name__']]
+        timetable.__name__ = data['__name__']
+
+        container[timetable.__name__] = timetable
+
+        timetable.periods, event = containedEvent(
+                factories[data['period_templates']](), timetable, 'periods')
+        notify(event)
+        timetable.periods.initTemplates()
+
+        timetable.time_slots, event = containedEvent(
+                factories[data['time_templates']](), timetable, 'time_slots')
+        notify(event)
+        timetable.time_slots.initTemplates()
+
+        name_chooser = INameChooser(timetable.periods.templates)
+        for entry in data['periods']:
+            day_id = entry['id']
+            day_title = day_id
+            if data['period_templates'] == 'week_days':
+                day_title = self.dows[int(day_id)]
+            day = DayTemplate(day_title)
+            if data['period_templates'] == 'week_days':
+                name = day_id
+            else:
+                name = name_chooser.chooseName('', day)
+            timetable.periods.templates[name] = day
+            p_chooser = INameChooser(day)
+            for period_entry in entry['periods']:
+                period = Period(title=period_entry['title'],
+                                activity_type=period_entry['activity'] or None)
+                p_name = p_chooser.chooseName('', period)
+                day[p_name] = period
+
+        name_chooser = INameChooser(timetable.time_slots.templates)
+        for entry in data['time_slots']:
+            day_id = entry['id']
+            day_title = day_id
+            if data['time_templates'] == 'week_days':
+                day_title = self.dows[int(day_id)]
+            day = DayTemplate(day_title)
+            if data['time_templates'] == 'week_days':
+                name = day_id
+            else:
+                name = name_chooser.chooseName('', day)
+            timetable.time_slots.templates[name] = day
+            ts_chooser = INameChooser(day)
+            for ts_entry in entry['time_slots']:
+                time_slot = TimeSlot(
+                    ts_entry['starts'], ts_entry['duration'],
+                    activity_type=ts_entry['activity'] or None)
+                ts_name = ts_chooser.chooseName('', time_slot)
+                day[ts_name] = time_slot
 
     def getWeeklyDayId(self, sh, row, col):
-        if sh.cell_value(rowx=row, colx=col) == 'default':
-            day_id = 'default'
+        value, found = self.getCellAndFound(sh, row, col)
+        if not found:
+            self.error(row, col, ERROR_WEEKLY_DAY_ID)
+            return None
+
+        if isinstance(value, float):
+            if int(value) == value:
+                value = int(value)
         else:
-            day_id, found, valid = self.getIntFoundValid(sh, row, col)
-            if valid and day_id not in range(7):
-                self.error(row, col, ERROR_WEEKLY_DAY_ID)
-        return day_id
+            try:
+                value = self.dows.index(value)
+            except ValueError:
+                self.errors.append("Unrecognised day id %s" % value)
+
+            if isinstance(value, float):
+                if int(value) != value:
+                    self.error(row, col, ERROR_NOT_INT)
+                else:
+                    value = int(value)
+        return unicode(value)
 
     def import_school_timetable(self, sh, row):
         num_errors = len(self.errors)
@@ -442,47 +533,106 @@ class SchoolTimetableImporter(ImporterBase):
         data['title'] = self.getRequiredTextFromCell(sh, row, 1)
         data['__name__'] = self.getRequiredTextFromCell(sh, row+1, 1)
         data['school_year'] = self.getRequiredTextFromCell(sh, row+2, 1)
-        data['model'] = self.getRequiredTextFromCell(sh, row+3, 1)
+        data['period_templates'] = self.getRequiredTextFromCell(sh, row+3, 1)
+        data['time_templates'] = self.getRequiredTextFromCell(sh, row+4, 1)
         if num_errors < len(self.errors):
             return
 
         num_errors = len(self.errors)
         if data['school_year'] not in ISchoolYearContainer(self.context):
             self.error(row + 2, 1, ERROR_INVALID_SCHOOL_YEAR)
-        if queryUtility(ITimetableModelFactory, data['model']) is None:
+
+        factories = dict(self.day_templates)
+        if data['period_templates'] not in factories:
             self.error(row + 3, 1, ERROR_TIMETABLE_MODEL)
+        if data['time_templates'] not in factories:
+            self.error(row + 4, 1, ERROR_TIMETABLE_MODEL)
+
         if num_errors < len(self.errors):
             return
 
         num_errors = len(self.errors)
         row += 5
-        if self.getCellValue(sh, row, 0, '') == 'Day Templates':
-            data['templates'] = []
+
+        row += 1
+        if self.getCellValue(sh, row, 0, '').lower() == 'days':
+            data['periods'] = []
             row += 1
-            for row in range(row, sh.nrows):
+
+            while row < sh.nrows:
                 if sh.cell_value(rowx=row, colx=0) == '':
                     break
 
-                if data['model'] == 'WeeklyTimetableModel':
+                if data['period_templates'] == 'week_days':
                     day_id = self.getWeeklyDayId(sh, row, 0)
                 else:
                     day_id = self.getRequiredTextFromCell(sh, row, 0)
 
-                if day_id in [day['id'] for day in data['templates']]:
+                if day_id in [day['id'] for day in data['periods']]:
                     self.error(row, 0, ERROR_DUPLICATE_DAY_ID)
 
                 periods = []
-                for col in range(1, sh.ncols):
-                    cell = sh.cell_value(rowx=row, colx=col)
+                col = 1
+                while True:
+                    cell = self.getCellValue(sh, row, col, default='')
+                    if cell == '':
+                        break
+                    activity = sh.cell_value(rowx=row+1, colx=col)
+                    if cell in periods:
+                        self.error(row, col, ERROR_DUPLICATE_PERIOD)
+                    else:
+                        periods.append({'title': cell,
+                                        'activity': activity})
+                    col += 1
+                row += 2
+
+                data['periods'].append({
+                        'id': day_id,
+                        'periods': periods,
+                        })
+        else:
+            self.errors.append(format_message(
+                ERROR_HAS_NO_DAYS,
+                mapping={'title': data['title'], 'row': row + 1}
+                ))
+
+        row += 1
+        if self.getCellValue(sh, row, 0, '').lower() == 'time schedule':
+            data['time_slots'] = []
+            row += 1
+
+            while row < sh.nrows:
+                if sh.cell_value(rowx=row, colx=0) == '':
+                    break
+
+                if data['time_templates'] == 'week_days':
+                    day_id = self.getWeeklyDayId(sh, row, 0)
+                else:
+                    day_id = self.getRequiredTextFromCell(sh, row, 0)
+
+                if day_id in [day['id'] for day in data['time_slots']]:
+                    self.error(row, 0, ERROR_DUPLICATE_DAY_ID)
+
+                time_slots = []
+                col = 1
+                while True:
+                    cell = self.getCellValue(sh, row, col, default='')
                     if cell == '':
                         break
                     try:
-                        time_range = parse_time_range(cell)
+                        starts, duration = parse_time_range(cell)
                     except:
                         self.error(row, col, ERROR_TIME_RANGE)
                         continue
-                    periods.append(time_range)
-                data['templates'].append({'id': day_id, 'periods': periods})
+                    activity = self.getCellValue(sh, row+1, col, default='')
+                    time_slots.append({
+                            'starts': starts,
+                            'duration': duration,
+                            'activity': activity,
+                            })
+                    col += 1
+                data['time_slots'].append({'id': day_id, 'time_slots': time_slots})
+                row += 2
         else:
             self.errors.append(format_message(
                 ERROR_HAS_NO_DAY_TEMPLATES,
@@ -491,59 +641,8 @@ class SchoolTimetableImporter(ImporterBase):
         if num_errors < len(self.errors):
             return
 
-        row += 1
-        if self.getCellValue(sh, row, 0, '') == 'Days':
-            data['days'] = []
-            homeroom_start = sh.ncols
-            for col in range(2, sh.ncols):
-                if sh.cell_value(rowx=row, colx=col) == 'Homeroom periods':
-                    homeroom_start = col
-                    break
-            row += 1
-            for row in range(row, sh.nrows):
-                if sh.cell_value(rowx=row, colx=0) == '':
-                    break
-
-                if data['model'] == 'WeeklyTimetableModel':
-                    day_id = self.getWeeklyDayId(sh, row, 0)
-                else:
-                    day_id = self.getRequiredTextFromCell(sh, row, 0)
-
-                if day_id in [day['id'] for day in data['days']]:
-                    self.error(row, 0, ERROR_DUPLICATE_DAY_ID)
-                if day_id not in [day['id'] for day in data['templates']]:
-                    self.error(row, 0, ERROR_UNKNOWN_DAY_ID)
-
-                periods = []
-                for col in range(1, homeroom_start):
-                    cell = sh.cell_value(rowx=row, colx=col)
-                    if cell == '':
-                        break
-                    if cell in periods:
-                        self.error(row, col, ERROR_DUPLICATE_PERIOD)
-                    else:
-                        periods.append(cell)
-                homeroom_periods = []
-                for col in range(homeroom_start, sh.ncols):
-                    cell = sh.cell_value(rowx=row, colx=col)
-                    if cell == '':
-                        break
-                    if cell in homeroom_periods:
-                        self.error(row, col, ERROR_DUPLICATE_HOMEROOM_PERIOD)
-                    else:
-                        homeroom_periods.append(cell)
-                data['days'].append({'id': day_id,
-                                     'periods': periods,
-                                     'homeroom_periods': homeroom_periods})
-        else:
-            self.errors.append(format_message(
-                ERROR_HAS_NO_DAYS,
-                mapping={'title': data['title'], 'row': row + 1}
-                ))
-
         if not self.errors:
-            school_timetable = self.createSchoolTimetable(data)
-            self.addSchoolTimetable(school_timetable, data)
+            self.createSchoolTimetable(data)
 
     def process(self):
         sh = self.sheet
@@ -557,8 +656,11 @@ class ResourceImporter(ImporterBase):
     sheet_name = 'Resources'
 
     def createResource(self, data):
-        res_types = {'Location': Location,
-                     'Resource': Resource}
+        res_types = {
+            'Location': Location,
+            'Equipment': Equipment,
+            'Resource': Resource,
+            }
         res_factory = res_types[data['type']]
         resource = res_factory(data['title'])
         resource.__name__ = data['__name__']
@@ -586,7 +688,7 @@ class ResourceImporter(ImporterBase):
             data['title'] = self.getRequiredTextFromCell(sh, row, 2)
             if num_errors < len(self.errors):
                 continue
-            if data['type'] not in ['Location', 'Resource']:
+            if data['type'] not in ['Location', 'Equipment', 'Resource']:
                 self.error(row, 1, ERROR_RESOURCE_TYPE)
                 continue
             resource = self.createResource(data)
@@ -666,18 +768,163 @@ class PersonImporter(ImporterBase):
 
             demographics = IDemographics(person)
             for n, field in enumerate(fields.values()):
-                if isinstance(field, DateFieldDescription):
-                    value = self.getDateFromCell(sh, row, n + 10, default=None)
+                if field.required:
+                    if isinstance(field, DateFieldDescription):
+                        value = self.getDateFromCell(sh, row, n + 10)
+                    elif isinstance(field, BoolFieldDescription):
+                        value = self.getRequiredBoolFromCell(sh, row, n + 10)
+                    else:
+                        value = self.getRequiredTextFromCell(sh, row, n + 10)
                 else:
-                    value = self.getTextFromCell(sh, row, n + 10)
-                if value == '':
-                    value = None
+                    if isinstance(field, DateFieldDescription):
+                        value = self.getDateFromCell(sh, row, n + 10,
+                                                     default=None)
+                    elif isinstance(field, BoolFieldDescription):
+                        value = self.getBoolFromCell(sh, row, n + 10)
+                    else:
+                        value = self.getTextFromCell(sh, row, n + 10)
+                    if value == '':
+                        value = None
                 demographics[field.name] = value
 
             if num_errors == len(self.errors):
                 self.addPerson(person, data)
 
 
+class ContactPersonImporter(ImporterBase):
+
+    sheet_name = 'Contact Persons'
+
+    def applyData(self, contact, data):
+        if data['__name__'] not in ISchoolToolApplication(None)['persons']:
+            contact.prefix = data['prefix']
+            contact.first_name = data['first_name']
+            contact.middle_name = data['middle_name']
+            contact.last_name = data['last_name']
+            contact.suffix = data['suffix']
+        contact.address_line_1 = data['address_line_1']
+        contact.address_line_2 = data['address_line_2']
+        contact.city = data['city']
+        contact.state = data['state']
+        contact.country = data['country']
+        contact.postal_code = data['postal_code']
+        contact.home_phone = data['home_phone']
+        contact.work_phone = data['work_phone']
+        contact.mobile_phone = data['mobile_phone']
+        contact.email = data['email']
+        contact.language = data['language']
+
+    def establishContact(self, data):
+        app = ISchoolToolApplication(None)
+        persons = app['persons']
+        contacts = IContactContainer(app)
+        name = data['__name__']
+        if name in persons:
+            person = persons[name]
+            contact = IContact(person)
+            self.applyData(contact, data)
+        elif name in contacts:
+            contact = contacts[name]
+            self.applyData(contact, data)
+        else:
+            contact = Contact()
+            self.applyData(contact, data)
+            contacts[name] = contact
+
+    def process(self):
+        sh = self.sheet
+        persons = ISchoolToolApplication(None)['persons']
+        for row in range(1, sh.nrows):
+            if sh.cell_value(rowx=row, colx=0) == '':
+                break
+
+            num_errors = len(self.errors)
+            data = {}
+
+            data['__name__'] = self.getRequiredTextFromCell(sh, row, 0)
+            self.validateUnicode(data['__name__'], row, 0)
+            if num_errors == len(self.errors):
+                if data['__name__'] not in persons:
+                    data['prefix'] = self.getTextFromCell(sh, row, 1)
+                    data['first_name'] = self.getRequiredTextFromCell(sh, row, 2)
+                    data['middle_name'] = self.getTextFromCell(sh, row, 3)
+                    data['last_name'] = self.getRequiredTextFromCell(sh, row, 4)
+                    data['suffix'] = self.getTextFromCell(sh, row, 5)
+                else:
+                    for index in range(5):
+                        value, found = self.getCellAndFound(sh, row, index + 1)
+                        if value:
+                            self.error(row, index + 1,
+                                       ERROR_UNWANTED_CONTACT_DATA)
+
+            data['address_line_1'] = self.getTextFromCell(sh, row, 6)
+            data['address_line_2'] = self.getTextFromCell(sh, row, 7)
+            data['city'] = self.getTextFromCell(sh, row, 8)
+            data['state'] = self.getTextFromCell(sh, row, 9)
+            data['country'] = self.getTextFromCell(sh, row, 10)
+            data['postal_code'] = self.getTextFromCell(sh, row, 11)
+            data['home_phone'] = self.getTextFromCell(sh, row, 12)
+            data['work_phone'] = self.getTextFromCell(sh, row, 13)
+            data['mobile_phone'] = self.getTextFromCell(sh, row, 14)
+            data['email'] = self.getTextFromCell(sh, row, 15)
+            data['language'] = self.getTextFromCell(sh, row, 16)
+
+            if num_errors == len(self.errors):
+                self.establishContact(data)
+
+
+class ContactRelationshipImporter(ImporterBase):
+
+    sheet_name = 'Contact Relationships'
+
+    def process(self):
+        sh = self.sheet
+        app = ISchoolToolApplication(None)
+        persons = app['persons']
+        contacts = IContactContainer(app)
+        vocab = IContactPersonInfo['relationship'].vocabulary
+        for row in range(1, sh.nrows):
+            if sh.cell_value(rowx=row, colx=0) == '':
+                break
+
+            num_errors = len(self.errors)
+            data = {}
+
+            data['__name__'] = self.getRequiredTextFromCell(sh, row, 0)
+            self.validateUnicode(data['__name__'], row, 0)
+            if num_errors == len(self.errors):
+                if data['__name__'] not in persons:
+                    self.error(row, 0, ERROR_INVALID_PERSON_ID)
+                else:
+                    person = persons[data['__name__']]
+
+            current_errors = len(self.errors)
+            data['contact_name'] = self.getRequiredTextFromCell(sh, row, 1)
+            name = data['contact_name']
+            if current_errors == len(self.errors):
+                self.validateUnicode(name, row, 1)
+            if current_errors == len(self.errors):
+                if name in persons:
+                    contact = IContact(persons[name])
+                elif name in contacts:
+                    contact = contacts[name]
+                else:
+                    self.error(row, 1, ERROR_INVALID_CONTACT_ID)
+
+            data['relationship'] = self.getTextFromCell(sh, row, 2)
+            relationship = data['relationship']
+            if relationship and relationship not in vocab:
+                self.error(row, 2, ERROR_CONTACT_RELATIONSHIP)
+
+            if num_errors == len(self.errors):
+                info = ContactPersonInfo()
+                info.__parent__ = person
+                if relationship:
+                    info.relationship = relationship
+                if contact not in IContactable(person).contacts:
+                    IContactable(person).contacts.add(contact, info)
+
+
 class CourseImporter(ImporterBase):
 
     sheet_name = 'Courses'
@@ -759,51 +1006,65 @@ class SectionImporter(ImporterBase):
                     next_sections[section.__name__].previous = section
 
     def import_timetable(self, sh, row, section):
-        schemas = ITimetableSchemaContainer(ISchoolYear(section))
-        schema_id = self.getRequiredTextFromCell(sh, row, 1)
-        if schema_id not in schemas:
-            self.error(row, 0, ERROR_INVALID_SCHEMA_ID)
-            return
-        schema = schemas[schema_id]
-        timetable = schema.createTimetable(ITerm(section))
-        timetables = ITimetables(section).timetables
-        timetables[schema_id] = timetable
+        timetables = ITimetableContainer(ISchoolYear(section))
 
+        timetable_id = self.getRequiredTextFromCell(sh, row, 1)
+        if timetable_id not in timetables:
+            self.error(row, 0, ERROR_INVALID_SCHEMA_ID)
+            return row
+        timetable = timetables[timetable_id]
+
+        schedules = IScheduleContainer(section)
+
+        term = ITerm(section)
+        schedule = SelectedPeriodsSchedule(
+            timetable, term.first, term.last,
+            title=timetable.title, timezone=timetable.timezone)
         row += 1
-        course = list(section.courses)[0]
-        rc = self.context['resources']
-        resources = {}
-        for row in range(row + 1, sh.nrows):
+
+        collapse_periods = self.getTextFromCell(sh, row, 1, 'no')
+        schedule.consecutive_periods_as_one = bool(
+            collapse_periods.lower() == 'yes')
+
+        row += 2
+
+        for row in range(row, sh.nrows):
+
             if sh.cell_value(rowx=row, colx=0) == '':
                 break
-
             num_errors = len(self.errors)
-            day_id = self.getRequiredTextFromCell(sh, row, 0)
-            period_id = self.getRequiredTextFromCell(sh, row, 1)
-            resource_id = self.getTextFromCell(sh, row, 2)
+            day_title = self.getRequiredTextFromCell(sh, row, 0)
+            period_title = self.getRequiredTextFromCell(sh, row, 1)
             if num_errors < len(self.errors):
                 continue
 
-            if day_id not in schema.day_ids:
+            day = None
+            for tt_day in timetable.periods.templates.values():
+                if tt_day.title == day_title:
+                    day = tt_day
+                    break
+            if day is None:
                 self.error(row, 0, ERROR_INVALID_DAY_ID)
-            elif period_id not in schema[day_id].periods:
+                continue
+
+            period = None
+            for tt_period in day.values():
+                if tt_period.title == period_title:
+                    period = tt_period
+            if period is None:
                 self.error(row, 1, ERROR_INVALID_PERIOD_ID)
-            if resource_id and resource_id not in rc:
-                self.error(row, 2, ERROR_INVALID_RESOURCE_ID)
+                continue
+
+            schedule.addPeriod(period)
+
             if num_errors < len(self.errors):
                 continue
 
-            if resource_id:
-                resources[day_id, period_id] = resource_id
-            act = TimetableActivity(title=course.title, owner=section)
-            timetable[day_id].add(period_id, act)
+        s_chooser = INameChooser(schedules)
+        name = s_chooser.chooseName('', schedule)
+        schedules[name] = schedule
 
-        for event in ISchoolToolCalendar(section):
-            if ITimetableCalendarEvent.providedBy(event):
-                resource_id = resources.get((event.day_id, event.period_id))
-                if resource_id:
-                    resource = rc[resource_id]
-                    event.bookResource(removeSecurityProxy(resource))
+        return row
 
     def import_timetables(self, sh, row, section):
         for row in range(row, sh.nrows):
@@ -1013,6 +1274,10 @@ class MegaImporter(BrowserView):
 
         wb = xlrd.open_workbook(file_contents=xlsfile.read())
 
+        if wb is None:
+            return
+
+
         sp = transaction.savepoint(optimistic=True)
 
         importers = [SchoolYearImporter,
@@ -1020,6 +1285,8 @@ class MegaImporter(BrowserView):
                      SchoolTimetableImporter,
                      ResourceImporter,
                      PersonImporter,
+                     ContactPersonImporter,
+                     ContactRelationshipImporter,
                      CourseImporter,
                      SectionImporter,
                      GroupImporter]
@@ -1031,7 +1298,27 @@ class MegaImporter(BrowserView):
 
         if self.errors:
             sp.rollback()
+        else:
+            self.request.response.redirect(self.nextURL())
+
+    def nextURL(self):
+        return self.request.URL
 
     def displayErrors(self):
         return self.errors[:25]
 
+
+
+class FlourishMegaImporter(flourish.page.Page, MegaImporter):
+    __init__ = MegaImporter.__init__
+    update = MegaImporter.update
+
+    def nextURL(self):
+        url = absoluteURL(self.context, self.request)
+        return '%s/manage' % url
+
+    def update(self):
+        if "UPDATE_CANCEL" in self.request:
+            self.request.response.redirect(self.nextURL())
+            return
+        return MegaImporter.update(self)
